@@ -24,6 +24,7 @@
 #include "config.h"
 #include "gdrive.h"
 #include "gsimpleasyncresult.h"
+#include "gthemedicon.h"
 #include "gasyncresult.h"
 #include "gioerror.h"
 #include "glibintl.h"
@@ -174,6 +175,35 @@ g_drive_get_icon (GDrive *drive)
   return (* iface->get_icon) (drive);
 }
 
+/**
+ * g_drive_get_symbolic_icon:
+ * @drive: a #GDrive.
+ * 
+ * Gets the icon for @drive.
+ * 
+ * Returns: (transfer full): symbolic #GIcon for the @drive.
+ *    Free the returned object with g_object_unref().
+ *
+ * Since: 2.34
+ **/
+GIcon *
+g_drive_get_symbolic_icon (GDrive *drive)
+{
+  GDriveIface *iface;
+  GIcon *ret;
+
+  g_return_val_if_fail (G_IS_DRIVE (drive), NULL);
+
+  iface = G_DRIVE_GET_IFACE (drive);
+
+  if (iface->get_symbolic_icon != NULL)
+    ret = iface->get_symbolic_icon (drive);
+  else
+    ret = g_themed_icon_new_with_default_fallbacks ("drive-removable-media-symbolic");
+
+  return ret;
+}
+
 /**
  * g_drive_has_volumes:
  * @drive: a #GDrive.
