@@ -579,6 +579,12 @@ g_menu_model_get_item_attribute_value (GMenuModel         *model,
  * type, then the positional parameters are ignored and %FALSE is
  * returned.
  *
+ * This function is a mix of g_menu_model_get_item_attribute_value() and
+ * g_variant_get(), followed by a g_variant_unref().  As such,
+ * @format_string must make a complete copy of the data (since the
+ * #GVariant may go away after the call to g_variant_unref()).  In
+ * particular, no '&amp;' characters are allowed in @format_string.
+ *
  * Returns: %TRUE if the named attribute was found with the expected
  *     type
  *
@@ -591,16 +597,20 @@ g_menu_model_get_item_attribute (GMenuModel  *model,
                                  const gchar *format_string,
                                  ...)
 {
-  const GVariantType *expected_type;
   GVariant *value;
   va_list ap;
 
-  expected_type = NULL; /* XXX devine the type, ensure no '&' */
+  value = g_menu_model_get_item_attribute_value (model, item_index, attribute, NULL);
 
-  value = g_menu_model_get_item_attribute_value (model, item_index, attribute, expected_type);
   if (value == NULL)
     return FALSE;
 
+  if (!g_variant_check_format_string (value, format_string, TRUE))
+    {
+      g_variant_unref (value);
+      return FALSE;
+    }
+
   va_start (ap, format_string);
   g_variant_get_va (value, format_string, NULL, &ap);
   g_variant_unref (value);
@@ -714,8 +724,8 @@ struct _GMenuAttributeIterPrivate
  *
  * If successful, @name and @value are set to the name and value of the
  * attribute that has just been advanced to.  At this point,
- * g_menu_item_get_name() and g_menu_item_get_value() will return the
- * same values again.
+ * g_menu_attribute_iter_get_name() and g_menu_attribute_iter_get_value() will
+ * return the same values again.
  *
  * The value returned in @name remains valid for as long as the iterator
  * remains at the current position.  The value returned in @value must
@@ -871,7 +881,7 @@ struct _GMenuLinkIterPrivate
  *
  * If successful, @out_link and @value are set to the name and #GMenuModel
  * of the link that has just been advanced to.  At this point,
- * g_menu_item_get_name() and g_menu_item_get_value() will return the
+ * g_menu_link_iter_get_name() and g_menu_link_iter_get_value() will return the
  * same values again.
  *
  * The value returned in @out_link remains valid for as long as the iterator
