@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors: Christian Kellner <gicmo@gnome.org>
  *          Samuel Cormier-Iijima <sciyoshi@gmail.com>
@@ -24,9 +26,8 @@
 #include <string.h>
 
 #include "gunixsocketaddress.h"
-#include "gsocketconnectable.h"
 #include "glibintl.h"
-#include "gnetworking.h"
+#include "gnetworkingprivate.h"
 
 
 /**
@@ -45,17 +46,18 @@
  * errors. You can use g_unix_socket_address_abstract_names_supported()
  * to see if abstract names are supported.
  *
- * Note that `<gio/gunixsocketaddress.h>` belongs to the UNIX-specific GIO
- * interfaces, thus you have to use the `gio-unix-2.0.pc` pkg-config file
- * when using it.
+ * Note that <filename>&lt;gio/gunixsocketaddress.h&gt;</filename> belongs to
+ * the UNIX-specific GIO interfaces, thus you have to use the
+ * <filename>gio-unix-2.0.pc</filename> pkg-config file when using it.
  */
 
 /**
  * GUnixSocketAddress:
  *
  * A UNIX-domain (local) socket address, corresponding to a
- * struct sockaddr_un.
+ * <type>struct sockaddr_un</type>.
  */
+G_DEFINE_TYPE (GUnixSocketAddress, g_unix_socket_address, G_TYPE_SOCKET_ADDRESS);
 
 enum
 {
@@ -77,14 +79,6 @@ struct _GUnixSocketAddressPrivate
   GUnixSocketAddressType address_type;
 };
 
-static void   g_unix_socket_address_connectable_iface_init (GSocketConnectableIface *iface);
-static gchar *g_unix_socket_address_connectable_to_string  (GSocketConnectable      *connectable);
-
-G_DEFINE_TYPE_WITH_CODE (GUnixSocketAddress, g_unix_socket_address, G_TYPE_SOCKET_ADDRESS,
-                         G_ADD_PRIVATE (GUnixSocketAddress)
-                         G_IMPLEMENT_INTERFACE (G_TYPE_SOCKET_CONNECTABLE,
-                                                g_unix_socket_address_connectable_iface_init))
-
 static void
 g_unix_socket_address_set_property (GObject      *object,
 				    guint         prop_id,
@@ -123,15 +117,26 @@ g_unix_socket_address_set_property (GObject      *object,
       break;
 
     case PROP_ABSTRACT:
-      /* Only set it if it's not the default... */
+      /* If the caller already set PROP_ADDRESS_TYPE, don't let the
+       * default value of PROP_ABSTRACT overwrite it.
+       */
+      if (address->priv->address_type != G_UNIX_SOCKET_ADDRESS_INVALID)
+	return;
+
       if (g_value_get_boolean (value))
-       address->priv->address_type = G_UNIX_SOCKET_ADDRESS_ABSTRACT_PADDED;
+	address->priv->address_type = G_UNIX_SOCKET_ADDRESS_ABSTRACT_PADDED;
+      else
+	address->priv->address_type = G_UNIX_SOCKET_ADDRESS_PATH;
       break;
 
     case PROP_ADDRESS_TYPE:
-      /* Only set it if it's not the default... */
-      if (g_value_get_enum (value) != G_UNIX_SOCKET_ADDRESS_PATH)
-        address->priv->address_type = g_value_get_enum (value);
+      /* If the caller already set PROP_ABSTRACT, don't let the
+       * default value of PROP_ADDRESS_TYPE overwrite it.
+       */
+      if (address->priv->address_type != G_UNIX_SOCKET_ADDRESS_INVALID)
+	return;
+
+      address->priv->address_type = g_value_get_enum (value);
       break;
 
     default:
@@ -254,6 +259,8 @@ g_unix_socket_address_class_init (GUnixSocketAddressClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GSocketAddressClass *gsocketaddress_class = G_SOCKET_ADDRESS_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GUnixSocketAddressPrivate));
+
   gobject_class->set_property = g_unix_socket_address_set_property;
   gobject_class->get_property = g_unix_socket_address_get_property;
 
@@ -306,57 +313,16 @@ g_unix_socket_address_class_init (GUnixSocketAddressClass *klass)
 						      G_PARAM_STATIC_STRINGS));
 }
 
-static void
-g_unix_socket_address_connectable_iface_init (GSocketConnectableIface *iface)
-{
-  GSocketConnectableIface *parent_iface = g_type_interface_peek_parent (iface);
-
-  iface->enumerate = parent_iface->enumerate;
-  iface->proxy_enumerate = parent_iface->proxy_enumerate;
-  iface->to_string = g_unix_socket_address_connectable_to_string;
-}
-
-static gchar *
-g_unix_socket_address_connectable_to_string (GSocketConnectable *connectable)
-{
-  GUnixSocketAddress *ua;
-  GString *out;
-  const gchar *path;
-  gsize path_len, i;
-
-  ua = G_UNIX_SOCKET_ADDRESS (connectable);
-
-  /* Anonymous sockets have no path. */
-  if (ua->priv->address_type == G_UNIX_SOCKET_ADDRESS_ANONYMOUS)
-    return g_strdup ("anonymous");
-
-  path = g_unix_socket_address_get_path (ua);
-  path_len = g_unix_socket_address_get_path_len (ua);
-  out = g_string_sized_new (path_len);
-
-  /* Return the #GUnixSocketAddress:path, but with all non-printable characters
-   * (including nul bytes) escaped to hex. */
-  for (i = 0; i < path_len; i++)
-    {
-      guint8 c = path[i];
-
-      if (g_ascii_isprint (path[i]))
-        g_string_append_c (out, c);
-      else
-        g_string_append_printf (out, "\\x%02x", (guint) c);
-    }
-
-  return g_string_free (out, FALSE);
-}
-
 static void
 g_unix_socket_address_init (GUnixSocketAddress *address)
 {
-  address->priv = g_unix_socket_address_get_instance_private (address);
+  address->priv = G_TYPE_INSTANCE_GET_PRIVATE (address,
+					       G_TYPE_UNIX_SOCKET_ADDRESS,
+					       GUnixSocketAddressPrivate);
 
   memset (address->priv->path, 0, sizeof (address->priv->path));
   address->priv->path_len = -1;
-  address->priv->address_type = G_UNIX_SOCKET_ADDRESS_PATH;
+  address->priv->address_type = G_UNIX_SOCKET_ADDRESS_INVALID;
 }
 
 /**
@@ -412,9 +378,6 @@ g_unix_socket_address_new_abstract (const gchar *path,
  * If @type is %G_UNIX_SOCKET_ADDRESS_PATH, this is equivalent to
  * calling g_unix_socket_address_new().
  *
- * If @type is %G_UNIX_SOCKET_ADDRESS_ANONYMOUS, @path and @path_len will be
- * ignored.
- *
  * If @path_type is %G_UNIX_SOCKET_ADDRESS_ABSTRACT, then @path_len
  * bytes of @path will be copied to the socket's path, and only those
  * bytes will be considered part of the name. (If @path_len is -1,
@@ -429,7 +392,7 @@ g_unix_socket_address_new_abstract (const gchar *path,
  * zero-padded buffer will be considered the name. (As above, if
  * @path_len is -1, then @path is assumed to be NUL-terminated.) In
  * this case, g_socket_address_get_native_size() will always return
- * the full size of a `struct sockaddr_un`, although
+ * the full size of a <literal>struct sockaddr_un</literal>, although
  * g_unix_socket_address_get_path_len() will still return just the
  * length of @path.
  *
