@@ -121,16 +121,6 @@ static const GDBusInterfaceVTable interface_vtable =
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static void
-connection_closed (GDBusConnection *connection,
-                   gboolean remote_peer_vanished,
-                   GError *Error,
-                   gpointer user_data)
-{
-  g_print ("Client disconnected.\n");
-  g_object_unref (connection);
-}
-
 static gboolean
 on_new_connection (GDBusServer *server,
                    GDBusConnection *connection,
@@ -154,7 +144,6 @@ on_new_connection (GDBusServer *server,
            g_dbus_connection_get_capabilities (connection) & G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING);
 
   g_object_ref (connection);
-  g_signal_connect (connection, "closed", G_CALLBACK (connection_closed), NULL);
   registration_id = g_dbus_connection_register_object (connection,
                                                        "/org/gtk/GDBus/TestObject",
                                                        introspection_data->interfaces[0],
@@ -169,74 +158,6 @@ on_new_connection (GDBusServer *server,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static gboolean
-allow_mechanism_cb (GDBusAuthObserver *observer,
-                    const gchar *mechanism,
-                    G_GNUC_UNUSED gpointer user_data)
-{
-  /*
-   * In a production GDBusServer that only needs to work on modern Unix
-   * platforms, consider requiring EXTERNAL (credentials-passing),
-   * which is the recommended authentication mechanism for AF_UNIX
-   * sockets:
-   *
-   * if (g_strcmp0 (mechanism, "EXTERNAL") == 0)
-   *   return TRUE;
-   *
-   * return FALSE;
-   *
-   * For this example we accept everything.
-   */
-
-  g_print ("Considering whether to accept %s authentication...\n", mechanism);
-  return TRUE;
-}
-
-static gboolean
-authorize_authenticated_peer_cb (GDBusAuthObserver *observer,
-                                 G_GNUC_UNUSED GIOStream *stream,
-                                 GCredentials *credentials,
-                                 G_GNUC_UNUSED gpointer user_data)
-{
-  gboolean authorized = FALSE;
-
-  g_print ("Considering whether to authorize authenticated peer...\n");
-
-  if (credentials != NULL)
-    {
-      GCredentials *own_credentials;
-      gchar *credentials_string = NULL;
-
-      credentials_string = g_credentials_to_string (credentials);
-      g_print ("Peer's credentials: %s\n", credentials_string);
-      g_free (credentials_string);
-
-      own_credentials = g_credentials_new ();
-
-      credentials_string = g_credentials_to_string (own_credentials);
-      g_print ("Server's credentials: %s\n", credentials_string);
-      g_free (credentials_string);
-
-      if (g_credentials_is_same_user (credentials, own_credentials, NULL))
-        authorized = TRUE;
-
-      g_object_unref (own_credentials);
-    }
-
-  if (!authorized)
-    {
-      /* In most servers you'd want to reject this, but for this example
-       * we allow it. */
-      g_print ("A server would often not want to authorize this identity\n");
-      g_print ("Authorizing it anyway for demonstration purposes\n");
-      authorized = TRUE;
-    }
-
-  return authorized;
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 int
 main (int argc, char *argv[])
 {
@@ -256,6 +177,8 @@ main (int argc, char *argv[])
 
   ret = 1;
 
+  g_type_init ();
+
   opt_address = NULL;
   opt_server = FALSE;
   opt_allow_anonymous = FALSE;
@@ -289,7 +212,6 @@ main (int argc, char *argv[])
 
   if (opt_server)
     {
-      GDBusAuthObserver *observer;
       GDBusServer *server;
       gchar *guid;
       GMainLoop *loop;
@@ -301,20 +223,14 @@ main (int argc, char *argv[])
       if (opt_allow_anonymous)
         server_flags |= G_DBUS_SERVER_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS;
 
-      observer = g_dbus_auth_observer_new ();
-      g_signal_connect (observer, "allow-mechanism", G_CALLBACK (allow_mechanism_cb), NULL);
-      g_signal_connect (observer, "authorize-authenticated-peer", G_CALLBACK (authorize_authenticated_peer_cb), NULL);
-
       error = NULL;
       server = g_dbus_server_new_sync (opt_address,
                                        server_flags,
                                        guid,
-                                       observer,
+                                       NULL, /* GDBusAuthObserver */
                                        NULL, /* GCancellable */
                                        &error);
       g_dbus_server_start (server);
-
-      g_object_unref (observer);
       g_free (guid);
 
       if (server == NULL)
@@ -359,8 +275,7 @@ main (int argc, char *argv[])
                "Negotiated capabilities: unix-fd-passing=%d\n",
                g_dbus_connection_get_capabilities (connection) & G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING);
 
-      greeting = g_strdup_printf ("Hey, it's %" G_GINT64_FORMAT " already!",
-                                  g_get_real_time () / G_USEC_PER_SEC);
+      greeting = g_strdup_printf ("Hey, it's %" G_GUINT64_FORMAT " already!", (guint64) time (NULL));
       value = g_dbus_connection_call_sync (connection,
                                            NULL, /* bus_name */
                                            "/org/gtk/GDBus/TestObject",
