@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,9 +30,7 @@
 #include "config.h"
 
 /* we know we are deprecated here, no need for warnings */
-#ifndef GLIB_DISABLE_DEPRECATION_WARNINGS
 #define GLIB_DISABLE_DEPRECATION_WARNINGS
-#endif
 
 #include "grel.h"
 
@@ -98,7 +96,7 @@ typedef struct _GRealTuples        GRealTuples;
  * GRelation:
  *
  * The #GRelation struct is an opaque data structure to represent a
- * [Relation][glib-Relations-and-Tuples]. It should
+ * <link linkend="glib-Relations-and-Tuples">Relation</link>. It should
  * only be accessed via the following functions.
  **/
 struct _GRelation
@@ -187,12 +185,11 @@ tuple_equal (gint fields)
 /**
  * g_relation_new:
  * @fields: the number of fields.
+ * @Returns: a new #GRelation.
  *
  * Creates a new #GRelation with the given number of fields. Note that
  * currently the number of fields must be 2.
  *
- * Returns: a new #GRelation.
- *
  * Deprecated: 2.26: Rarely used API
  **/
 GRelation*
@@ -378,12 +375,11 @@ g_relation_delete_tuple (gpointer tuple_key,
  * @relation: a #GRelation.
  * @key: the value to compare with.
  * @field: the field of each record to match.
+ * @Returns: the number of records deleted.
  *
  * Deletes any records from a #GRelation that have the given key value
  * in the given field.
  *
- * Returns: the number of records deleted.
- *
  * Deprecated: 2.26: Rarely used API
  **/
 gint
@@ -443,13 +439,12 @@ g_relation_select_tuple (gpointer tuple_key,
  * @relation: a #GRelation.
  * @key: the value to compare with.
  * @field: the field of each record to match.
+ * @Returns: the records (tuples) that matched.
  *
  * Returns all of the tuples which have the given key in the given
  * field. Use g_tuples_index() to access the returned records. The
  * returned records should be freed with g_tuples_destroy().
  *
- * Returns: the records (tuples) that matched.
- *
  * Deprecated: 2.26: Rarely used API
  **/
 GTuples*
@@ -491,12 +486,11 @@ g_relation_select (GRelation     *relation,
  * @relation: a #GRelation.
  * @key: the value to compare with.
  * @field: the field of each record to match.
+ * @Returns: the number of matches.
  *
  * Returns the number of tuples in a #GRelation that have the given
  * value in the given field.
  *
- * Returns: the number of matches.
- *
  * Deprecated: 2.26: Rarely used API
  **/
 gint
@@ -526,13 +520,12 @@ g_relation_count (GRelation     *relation,
  * @relation: a #GRelation.
  * @...: the fields of the record to compare. The number must match
  *       the number of fields in the #GRelation.
+ * @Returns: %TRUE if a record matches.
  *
  * Returns %TRUE if a record with the given values exists in a
  * #GRelation. Note that the values are compared directly, so that, for
  * example, two copies of the same string will not match.
  *
- * Returns: %TRUE if a record matches.
- *
  * Deprecated: 2.26: Rarely used API
  **/
 gboolean
@@ -585,13 +578,12 @@ g_tuples_destroy (GTuples *tuples0)
  * @tuples: the tuple data, returned by g_relation_select().
  * @index_: the index of the record.
  * @field: the field to return.
+ * @Returns: the field of the record.
  *
  * Gets a field from the records returned by g_relation_select(). It
  * returns the given field of the record at the given index. The
  * returned value should not be changed.
  *
- * Returns: the field of the record.
- *
  * Deprecated: 2.26: Rarely used API
  **/
 gpointer
