@@ -35,6 +35,7 @@
 #include "gstring.h"
 #include "gtestutils.h"
 #include "glibintl.h"
+#include "gthread.h"
 
 /**
  * SECTION:markup
@@ -82,11 +83,7 @@
  * </itemizedlist>
  */
 
-GQuark
-g_markup_error_quark (void)
-{
-  return g_quark_from_static_string ("g-markup-error-quark");
-}
+G_DEFINE_QUARK (g-markup-error-quark, g_markup_error)
 
 typedef enum
 {
@@ -2341,7 +2338,7 @@ g_markup_vprintf_escaped (const gchar *format,
    * then use the normal g_strdup_vprintf() to format the arguments
    * with the two new format strings. By comparing the results,
    * we can figure out what segments of the output come from
-   * the the original format string, and what from the arguments,
+   * the original format string, and what from the arguments,
    * and thus know what portions of the string to escape.
    *
    * For instance, for:
