@@ -40,6 +40,7 @@
 #include <sys/utime.h>
 #else
 #include <utime.h>
+#include <errno.h>
 #endif
 
 #include "gstdio.h"
@@ -209,7 +210,11 @@ g_open (const gchar *filename,
   errno = save_errno;
   return retval;
 #else
-  return open (filename, flags, mode);
+  int fd;
+  do
+    fd = open (filename, flags, mode);
+  while (G_UNLIKELY (fd == -1 && errno == EINTR));
+  return fd;
 #endif
 }
 
@@ -737,7 +742,7 @@ g_fopen (const gchar *filename,
  * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @mode: a string describing the mode in which the file should be 
  *   opened
- * @stream: an existing stream which will be reused, or %NULL
+ * @stream: (allow-none): an existing stream which will be reused, or %NULL
  *
  * A wrapper for the POSIX freopen() function. The freopen() function
  * opens a file and associates it with an existing stream.
