@@ -1,7 +1,18 @@
+/*
+ * Copyright 2011 Red Hat, Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * See the included COPYING file for more information.
+ */
+
 #include <glib.h>
 
-int
-main (void)
+static void
+test_types (void)
 {
   guint u, u2;
   gint s, s2;
@@ -151,6 +162,11 @@ main (void)
   s2 = g_atomic_int_xor ((guint*)&s, 4);
   g_assert_cmpint (s2, ==, 12);
   g_assert_cmpint (s, ==, 8);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  s2 = g_atomic_int_exchange_and_add ((gint*)&s, 1);
+G_GNUC_END_IGNORE_DEPRECATIONS
+  g_assert_cmpint (s2, ==, 8);
+  g_assert_cmpint (s, ==, 9);
 
   g_atomic_pointer_set (&vp, 0);
   vp2 = g_atomic_pointer_get (&vp);
@@ -184,6 +200,63 @@ main (void)
   gs2 = g_atomic_pointer_xor (&gs, 4);
   g_assert (gs2 == 12);
   g_assert (gs == 8);
+}
 
-  return 0;
+#define THREADS 10
+#define ROUNDS 10000
+
+volatile gint bucket[THREADS];
+volatile gint atomic;
+
+static gpointer
+thread_func (gpointer data)
+{
+  gint idx = GPOINTER_TO_INT (data);
+  gint i;
+  gint d;
+
+  for (i = 0; i < ROUNDS; i++)
+    {
+      d = g_random_int_range (-10, 100);
+      bucket[idx] += d;
+      g_atomic_int_add (&atomic, d);
+      g_thread_yield ();
+    }
+
+  return NULL;
+}
+
+static void
+test_threaded (void)
+{
+  gint sum;
+  gint i;
+  GThread *threads[THREADS];
+
+  atomic = 0;
+  for (i = 0; i < THREADS; i++)
+    bucket[i] = 0;
+
+  for (i = 0; i < THREADS; i++)
+    threads[i] = g_thread_new ("atomic", thread_func, GINT_TO_POINTER (i));
+
+  for (i = 0; i < THREADS; i++)
+    g_thread_join (threads[i]);
+
+  sum = 0;
+  for (i = 0; i < THREADS; i++)
+    sum += bucket[i];
+
+  g_assert_cmpint (sum, ==, atomic);
+}
+
+int
+main (int argc, char **argv)
+{
+  g_test_init (&argc, &argv, NULL);
+
+  g_test_add_func ("/atomic/types", test_types);
+  g_test_add_func ("/atomic/threaded", test_threaded);
+
+  return g_test_run ();
 }
